/*
  MOD-MAG example.
  
  Reads the raw data from the
  module's registers.
  
  Values may need calibrating. Please
  refer to the MAG3110 Reference manual
  for detailed information.
  
  If you have any questions, email
  support@olimex.com
  
  https://www.olimex.com
  OLIMEX, 2013
*/

/* Pinguino standard I2C library */
#include "i2c.c"



/* Utilities */
   //WAIT Macro 
     #define WAIT wait_123()
     char ch123;
     void wait_123(){ ch123 = CDC.getKey(); }
   //END Wait Macro
/* Utilities */
     
void print_data();

void setup(){

// Open I2C communication to module
	I2C_init(I2C1, I2C_MASTER_MODE, I2C_SLEW_OFF);

// Press any key to start 
  WAIT;
  CDC.println("MOD-MAG Demo; press any key to continue...");
  WAIT;
}

void loop(){
  
  print_data();      
  CDC.println("");
  WAIT; //Press any key to read again
  
}

void print_data(){
  
	const static unsigned char RW_MASK  = 0x80;
	const static unsigned char SL_A     = 0x0E;
	const static unsigned char SL_READ  = 0x1D;
	const static unsigned char SL_WRITE = 0x1C;
 
	const static char  ACK = 0;
	const static char NACK = 1;
	
	const static unsigned char CTRL_REG1 = 0x10;
	const static unsigned char CTRL_REG2 = 0x11;
	
  //Update values to current data
  //This function uses the "Trigger Measurement" mode
  //Refer to the MAG3110 manual for detailed information
  //pages 12 and 17
  
  //Trigger Measurement by setting the TM bit in CTRL_REG1
  //Refer to page 12 and 17 of the MAG3110 manual for detailed information
  I2C_start(I2C1);
	I2C_writechar(I2C1,SL_WRITE);
	I2C_writechar(I2C1,CTRL_REG2);
	I2C_writechar(I2C1,0x80);
	I2C_stop(I2C1);	
   
	I2C_start(I2C1);
	I2C_writechar(I2C1,SL_WRITE);
	I2C_writechar(I2C1,CTRL_REG1);
	I2C_writechar(I2C1,0b00011010);
	I2C_stop(I2C1);	
  
	//Burst-read of the 6 bytes of data for X, Y and Z
	char buffer[6]={0};
	
	I2C_start(I2C1);
	I2C_writechar(I2C1,SL_WRITE);
	I2C_writechar(I2C1,0x01);
	I2C_stop(I2C1);	
	I2C_start(I2C1);
	I2C_writechar(I2C1,SL_READ);
  char i;
	for(i=0; i<5; i++)
		buffer[i] = I2C_readchar(I2C1,ACK);
		
	buffer[5] = I2C_readchar(I2C1,NACK);				
	I2C_stop(I2C1);	
	
	int16_t X=0, Y=0, Z=0;
  signed char temperature=0;
	X = buffer[0]; X<<=8; X |= buffer[1];
	Y = buffer[2]; Y<<=8; Y |= buffer[3];
	Z = buffer[4]; Z<<=8; Z |= buffer[5];	
  
	//Get Temperature
	delay(50);
	I2C_start(I2C1);
	I2C_writechar(I2C1,SL_WRITE);
	I2C_writechar(I2C1,0x0f);
	I2C_stop(I2C1);	
	I2C_start(I2C1);
	I2C_writechar(I2C1,SL_READ);
	temperature = I2C_readchar(I2C1,NACK);
	I2C_stop(I2C1);	  
  
  CDC.println("------ Raw Data(needs calibration, see MAG chip manual) ------");
  CDC.print("X: "); CDC.printf("%d \r\n",X); 
  CDC.print("Y: "); CDC.printf("%d \r\n",Y);
  CDC.print("Z: "); CDC.printf("%d \r\n",Z); 
  CDC.print("Temperature: "); CDC.printf("%d \r\n", temperature);
  CDC.println("------------------------------\r\n");
}
